@@ -1,16 +1,16 @@
 import pandas as pd
 
 from phc.easy.frame import Frame
-from phc.easy.patient_item import PatientItem
+from phc.easy.abstract.fhir_service_patient_item import FhirServicePatientItem
 
 
-class Condition(PatientItem):
+class Condition(FhirServicePatientItem):
     @staticmethod
     def table_name():
         return "condition"
 
     @staticmethod
-    def code_keys():
+    def code_fields():
         return [
             "meta.tag",
             "code.coding",
@@ -38,5 +38,6 @@ class Condition(PatientItem):
                 *expand_args.get("custom_columns", []),
                 Frame.codeable_like_column_expander("subject"),
                 Frame.codeable_like_column_expander("onsetPeriod"),
+                Frame.codeable_like_column_expander("context"),
             ],
         )
