@@ -1,16 +1,16 @@
 import pandas as pd
 
 from phc.easy.frame import Frame
-from phc.easy.patient_item import PatientItem
+from phc.easy.abstract.fhir_service_patient_item import FhirServicePatientItem
 
 
-class Goal(PatientItem):
+class Goal(FhirServicePatientItem):
     @staticmethod
     def table_name():
         return "goal"
 
     @staticmethod
-    def code_keys():
+    def code_fields():
         return ["meta.tag", "target.detailQuantity", "target.measure.coding"]
 
     @staticmethod
