@@ -4,17 +4,17 @@ import pandas as pd
 
 from phc.easy.auth import Auth
 from phc.easy.frame import Frame
-from phc.easy.patient_item import PatientItem
+from phc.easy.abstract.fhir_service_patient_item import FhirServicePatientItem
 from phc.easy.query import Query
 
 
-class Procedure(PatientItem):
+class Procedure(FhirServicePatientItem):
     @staticmethod
     def table_name():
         return "procedure"
 
     @staticmethod
-    def code_keys():
+    def code_fields():
         return ["meta.tag", "code.coding", "category.coding"]
 
     @staticmethod
@@ -30,6 +30,8 @@ class Procedure(PatientItem):
                 *expand_args.get("custom_columns", []),
                 Frame.codeable_like_column_expander("subject"),
                 Frame.codeable_like_column_expander("performedPeriod"),
+                Frame.codeable_like_column_expander("context"),
+                Frame.codeable_like_column_expander("managingOrganization"),
             ],
         }
 
