@@ -1,20 +1,26 @@
-from typing import Any, Callable, List, Union, NamedTuple, Tuple
-import pandas as pd
+import json
+import math
+from typing import Any, Callable, List, Optional, Tuple, Union
 
-from phc.services import Fhir
+import pandas as pd
 from phc.base_client import BaseClient
 from phc.easy.auth import Auth
-from phc.easy.query.ga4gh import recursive_execute_ga4gh
-from phc.easy.query.fhir_dsl_query import build_query
+from phc.easy.query.api_paging import clean_params, recursive_paging_api_call
+from phc.easy.query.fhir_aggregation import FhirAggregation
 from phc.easy.query.fhir_dsl import (
+    DEFAULT_SCROLL_SIZE,
     MAX_RESULT_SIZE,
-    recursive_execute_fhir_dsl,
     execute_single_fhir_dsl,
+    recursive_execute_fhir_dsl,
     tqdm,
     with_progress,
 )
-from phc.util.api_cache import APICache
-from phc.easy.query.fhir_aggregation import FhirAggregation
+from phc.easy.query.fhir_dsl_query import build_query
+from phc.easy.query.ga4gh import recursive_execute_ga4gh
+from phc.easy.util import extract_codes
+from phc.easy.util.api_cache import FHIR_DSL, APICache
+from phc.services import Fhir
+from toolz import identity
 
 
 class Query:
@@ -50,15 +56,7 @@ class Query:
         fhir = Fhir(auth.session())
 
         response = fhir.execute_es(
-            auth.project_id,
-            {
-                **query,
-                "limit": [
-                    {"type": "number", "value": 0},
-                    {"type": "number", "value": 1},
-                ],
-            },
-            scroll="true",
+            auth.project_id, build_query(query, page_size=1), scroll="true"
         )
 
         return response.data["hits"]["total"]["value"]
@@ -69,6 +67,7 @@ class Query:
         all_results: bool = False,
         auth_args: Auth = Auth.shared(),
         callback: Union[Callable[[Any, bool], None], None] = None,
+        max_pages: Union[int, None] = None,
         log: bool = False,
         **query_kwargs,
     ):
@@ -101,6 +100,9 @@ class Query:
                     if is_finished:
                         return "batch finished
 
+        max_pages : int
+            The number of pages to retrieve (useful if working with tons of records)
+
         log : bool = False
             Whether to log the elasticsearch query sent to the server
 
@@ -125,7 +127,7 @@ class Query:
         query = build_query(query, **query_kwargs)
 
         if log:
-            print(query)
+            print(json.dumps(query, indent=4))
 
         if FhirAggregation.is_aggregation_query(query):
             response = execute_single_fhir_dsl(query, auth_args=auth_args)
@@ -140,10 +142,7 @@ class Query:
                             {"type": "number", "value": 0},
                             # Make window size smaller than maximum to reduce
                             # pressure on API
-                            {
-                                "type": "number",
-                                "value": int(MAX_RESULT_SIZE / 2),
-                            },
+                            {"type": "number", "value": DEFAULT_SCROLL_SIZE},
                         ],
                         **query,
                     },
@@ -151,13 +150,133 @@ class Query:
                     progress=progress,
                     callback=callback,
                     auth_args=auth_args,
+                    max_pages=max_pages,
                 ),
             )
 
         return recursive_execute_fhir_dsl(
-            query, scroll=all_results, callback=callback, auth_args=auth_args,
+            query,
+            scroll=all_results,
+            callback=callback,
+            auth_args=auth_args,
+            max_pages=max_pages,
         )
 
+    @staticmethod
+    def execute_paging_api(
+        path: str,
+        params: dict = {},
+        http_verb: str = "GET",
+        transform: Callable[[pd.DataFrame], pd.DataFrame] = identity,
+        all_results: bool = False,
+        auth_args: Auth = Auth.shared(),
+        max_pages: Optional[int] = None,
+        page_size: Optional[int] = None,
+        log: bool = False,
+        raw: bool = False,
+        ignore_cache: bool = False,
+        progress: Optional[tqdm] = None,
+    ):
+        """Execute a API query that pages through results
+
+        See https://docs.us.lifeomic.com/api/?shell#lifeomic-core-api-genomics
+        for example
+
+        Attributes
+        ----------
+        path : str
+            The API path to hit
+            (Special tokens: `:project_id`)
+
+        params : dict
+            The parameters to include with request
+
+        http_verb : str
+            The HTTP method to use
+
+        all_results : bool = False
+            Retrieve sample of results (25) or entire set of records
+
+        auth_args : Auth, dict
+            Additional arguments for authentication
+
+        max_pages : int
+            The number of pages to retrieve (useful if working with tons of records)
+
+        page_size : int
+            The number of records to fetch per page
+
+        log : bool = False
+            Whether to log some diagnostic statements for debugging
+
+        progress : Optional[tqdm] = None
+            Override the given progress indicator
+
+        Examples
+        --------
+        >>> import phc.easy as phc
+        >>> phc.Auth.set({ 'account': '<your-account-name>' })
+        >>> phc.Project.set_current('My Project Name')
+        >>> phc.Query.execute_paging_api(
+                "genomics/projects/:project_id/tests",
+                params={
+                    "patientId": "<patient-uuid>"
+                }
+            )
+        """
+
+        auth = Auth(auth_args)
+
+        params = clean_params(params)
+        path = path.replace(":project_id", auth.project_id)
+        query = {"path": path, "method": http_verb, "params": params}
+
+        if all_results and page_size is None:
+            # Default to 100 if not provided but getting all results
+            page_size = 100
+
+        if log:
+            print(json.dumps(query, indent=4))
+
+        use_cache = (
+            (not ignore_cache)
+            and (not raw)
+            and all_results
+            and (max_pages is None)
+        )
+
+        if use_cache and APICache.does_cache_for_query_exist(query):
+            return APICache.load_cache_for_query(query)
+
+        callback = (
+            APICache.build_cache_callback(query, transform, nested_key=None)
+            if use_cache
+            else None
+        )
+
+        results = with_progress(
+            lambda: progress if progress is not None else tqdm(),
+            lambda progress: recursive_paging_api_call(
+                path,
+                params=params,
+                http_verb=http_verb,
+                callback=callback,
+                scroll=all_results or (max_pages is not None),
+                max_pages=max_pages,
+                page_size=page_size,
+                log=log,
+                auth_args=auth_args,
+                progress=progress,
+            ),
+        )
+
+        df = pd.DataFrame(results)
+
+        if raw:
+            return df
+
+        return transform(df)
+
     @staticmethod
     def execute_fhir_dsl_with_options(
         query: dict,
@@ -167,31 +286,39 @@ class Query:
         query_overrides: dict,
         auth_args: Auth,
         ignore_cache: bool,
+        max_pages: Union[int, None],
         log: bool = False,
         **query_kwargs,
     ):
         query = build_query({**query, **query_overrides}, **query_kwargs)
 
         if log:
-            print(query)
+            print(json.dumps(query, indent=4))
 
         use_cache = (
             (not ignore_cache)
             and (not raw)
             and (all_results or FhirAggregation.is_aggregation_query(query))
+            and (max_pages is None)
         )
 
-        if use_cache and APICache.does_cache_for_fhir_dsl_exist(query):
-            return APICache.load_cache_for_fhir_dsl(query)
+        if use_cache and APICache.does_cache_for_query_exist(
+            query, namespace=FHIR_DSL
+        ):
+            return APICache.load_cache_for_query(query, namespace=FHIR_DSL)
 
         callback = (
-            APICache.build_cache_fhir_dsl_callback(query, transform)
+            APICache.build_cache_callback(query, transform, namespace=FHIR_DSL)
             if use_cache
             else None
         )
 
         results = Query.execute_fhir_dsl(
-            query, all_results, auth_args, callback=callback
+            query,
+            all_results,
+            auth_args,
+            callback=callback,
+            max_pages=max_pages,
         )
 
         if isinstance(results, FhirAggregation):
@@ -212,8 +339,14 @@ class Query:
         return transform(df)
 
     @staticmethod
-    def get_codes(table_name: str, code_fields: List[str], **kwargs):
-        """Find FHIR codes for a given table
+    def get_codes(
+        table_name: str,
+        code_fields: List[str],
+        display_query: Optional[str] = None,
+        sample_size: Optional[int] = None,
+        **kwargs,
+    ):
+        """Find FHIR codes with a display for a given table
 
         Attributes
         ----------
@@ -223,8 +356,16 @@ class Query:
         code_fields : List[str]
             The fields of this table that contain a system, code, and display
 
+        display_query : Optional[str]
+            Part of the code's display to match (will try to extract full code
+            if passed)
+
+        sample_size : Optional[int]
+            Override the search size for finding codes (may miss codes on later
+            records)
+
         kwargs : dict
-            Arguments to pass to :func:`~phc.easy.query.Query.execute_composite_aggregations`
+            Arguments to pass to `phc.easy.query.Query.execute_composite_aggregations`
 
         Examples
         --------
@@ -246,26 +387,35 @@ class Query:
             frame["field"] = prefix
             return frame
 
+        if display_query is not None:
+            kwargs = {
+                **kwargs,
+                "query_overrides": {
+                    "where": {
+                        "type": "elasticsearch",
+                        "query": {
+                            "multi_match": {
+                                "query": display_query,
+                                "fields": [
+                                    f"{key}.display" for key in code_fields
+                                ],
+                            }
+                        },
+                    }
+                },
+            }
+
         results = Query.execute_composite_aggregations(
             table_name=table_name,
             key_sources_pairs=[
                 (
                     field,
                     [
-                        {
-                            "system": {
-                                "terms": {"field": f"{field}.system.keyword"}
-                            }
-                        },
-                        {"code": {"terms": {"field": f"{field}.code.keyword"}}},
                         {
                             "display": {
-                                "terms": {
-                                    "field": f"{field}.display.keyword",
-                                    "missing_bucket": True,
-                                }
+                                "terms": {"field": f"{field}.display.keyword"}
                             }
-                        },
+                        }
                     ],
                 )
                 for field in code_fields
@@ -273,13 +423,105 @@ class Query:
             **kwargs,
         )
 
-        return pd.concat(
-            [
-                agg_composite_to_frame(key, value)
-                for key, value in results.items()
-            ]
+        agg_result = (
+            pd.concat(
+                [
+                    agg_composite_to_frame(key, value)
+                    for key, value in results.items()
+                ]
+            )
+            .pipe(
+                lambda df: df
+                if len(df) == 0 or display_query is None
+                # Poor man's way to filter only matching codes (since Elasticsearch
+                # returns records which will include other codes)
+                else df[
+                    df["display"]
+                    .str.lower()
+                    .str.contains(display_query.lower())
+                ]
+            )
+            .pipe(
+                lambda df: pd.DataFrame()
+                if len(df) == 0
+                else df.sort_values("doc_count", ascending=False).reset_index(
+                    drop=True
+                )
+            )
         )
 
+        if display_query is None or len(agg_result) == 0:
+            return agg_result
+
+        min_count = sample_size or agg_result.doc_count.sum()
+        filtered_code_fields = agg_result.field.unique()
+
+        # Shortcut: If one result, we just need to get the other associated
+        # attributes of the code
+        if len(agg_result) == 1:
+            min_count = 1
+
+        code_results = Query.execute_fhir_dsl(
+            {
+                "type": "select",
+                "from": [{"table": table_name}],
+                "columns": [
+                    {
+                        "expr": {
+                            "type": "column_ref",
+                            "column": key.split(".")[0],
+                        }
+                    }
+                    for key in filtered_code_fields
+                ],
+                "where": {
+                    "type": "elasticsearch",
+                    "query": {
+                        "multi_match": {
+                            "query": display_query,
+                            "fields": [
+                                f"{key}.display" for key in filtered_code_fields
+                            ],
+                        }
+                    },
+                },
+            },
+            page_size=int(min_count % 9000),
+            max_pages=int(math.ceil(min_count / 9000)),
+            log=kwargs.get("log", False),
+        )
+
+        codes = extract_codes(
+            map(lambda d: d["_source"], code_results),
+            display_query,
+            code_fields,
+        )
+
+        if len(codes) == 0:
+            return codes
+
+        if len(codes) == codes.display.nunique():
+            # If display values are unique, then the counts from Elasticsearch
+            # are correct. We can therefore join them.
+            codes = (
+                codes.join(
+                    agg_result[["display", "doc_count"]].set_index("display"),
+                    on="display",
+                    how="outer",
+                )
+                .sort_values("doc_count", ascending=False)
+                .reset_index(drop=True)
+            )
+
+            if len(codes[codes.field.isnull()]) > 0:
+                print(
+                    "Records with missing system/code values were not retrieved."
+                )
+
+            return codes
+
+        return codes
+
     @staticmethod
     def execute_composite_aggregations(
         table_name: str,
@@ -359,9 +601,7 @@ class Query:
             table_name="observation",
             key_sources_pairs=[
                 ("meta.tag", [
-                    {"system": {"terms": {"field": "meta.tag.system.keyword"}}},
                     {"code": {"terms": {"field": "meta.tag.code.keyword"}}},
-                    {"display": {"terms": {"field": "meta.tag.display.keyword"}}},
                 ]),
                 ("code.coding", [
                     {"display": {"terms": {"field": "code.coding.display.keyword"}}}
